#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "loop.h"

void loopinfo(int fd) 
{
    struct loop_info info;

    if (ioctl(fd, LOOP_GET_STATUS, &info) < 0) {
        perror("loop_get_status");
        return;
    }
    
    printf("loop_get_status\n");
    printf("  lo_number = %u\n", info.lo_number);
    printf("  lo_encrypt_type = %u\n", info.lo_encrypt_type);
    printf("  lo_encrypt_key_size = %u\n", info.lo_encrypt_key_size);
    printf("  lo_flags = %#x\n", info.lo_flags);
}

void loopinfo64(int fd)
{
    int n;
    struct loop_info64 info64;

    if (ioctl(fd, LOOP_GET_STATUS64, &info64) < 0) {
        perror("loop_get_status64");
        return;
    }

    printf("loop_get_status64\n");
    printf("  lo_number = %u\n", info64.lo_number);
    printf("  lo_encrypt_type = %#x\n", info64.lo_encrypt_type);
    printf("  lo_encrypt_key_size = %u (%u)\n", info64.lo_encrypt_key_size, info64.lo_encrypt_key_size*8);
    printf("  lo_flags = %#x\n", info64.lo_flags);
    printf("  lo_crypt_name = '%s'\n", info64.lo_crypt_name);
    printf("  lo_encrypt_key = ");
    
    for (n=0; n < info64.lo_encrypt_key_size; n++)
        printf("%02x", info64.lo_encrypt_key[n]);

    printf(" ('%s')\n", info64.lo_encrypt_key);

    printf("  lo_file_name = '%s'\n", info64.lo_file_name);
}

int main(int argc, char **argv)
{
    int loopfd; 

    if (argc != 2) {
        printf("usage: loopinfo <device>\n");
        exit(1);
    }
    
    loopfd = open(argv[1], O_RDONLY);
    if (loopfd < 0) {
        perror(argv[1]);
        exit(1);
    }
 
    loopinfo(loopfd);
    loopinfo64(loopfd);

    close(loopfd);
    return 0;
}

